from django.utils.translation import gettext_lazy as _
from edc_constants.constants import (
    ABSENT,
    CLOSED,
    DEAD,
    GRADE3,
    GRADE4,
    NEW,
    NO,
    NO_EXAM,
    NOT_APPLICABLE,
    OPEN,
    OTHER,
    PATIENT,
    PENDING,
    PRESENT,
    PRESENT_WITH_REINFORCEMENT,
    YES,
)
from edc_visit_tracking.constants import MISSED_VISIT, SCHEDULED, UNSCHEDULED

from meta_ae.choices import INFORMANT_RELATIONSHIP
from meta_ae.constants import HOSPITAL_CLINIC

from .constants import (
    ALL_OF_THE_TIME,
    APPT,
    APPT_OTHER,
    GOOD_BIT_OF_THE_TIME,
    LITTLE_OF_THE_TIME,
    LIVE_AT_TERM,
    LIVE_PRETERM,
    MOST_OF_THE_TIME,
    NO_COMPLICATIONS,
    NONE_OF_THE_TIME,
    SOME_OF_THE_TIME,
)

ACTIVITY_CHOICES = (
    ("working", _("Working")),
    ("studying", _("Studying")),
    ("caring_for_children", _("Caring for children")),
    (OTHER, _("Other, please specify")),
)


CHILDCARE_CHOICES = (
    (NOT_APPLICABLE, _("Not applicable")),
    ("working", _("Working")),
    ("studying", _("Studying")),
    ("caring_for_children", _("Caring for children")),
    ("house_maintenance", _("House maintenance")),
    ("nothing", _("Nothing")),
    (OTHER, _("Other, specify")),
)

DELIVERY_INFO_SOURCE = (
    (PATIENT, _("Study participant")),
    (OTHER, _("Other")),
    (NOT_APPLICABLE, _("Not applicable")),
)

DELIVERY_LOCATIONS = (
    ("home", _("At home")),
    (HOSPITAL_CLINIC, _("Hospital / Clinic")),
    (OTHER, _("Other location, specify")),
    (NOT_APPLICABLE, _("Not applicable")),
)

DESCRIBE_HEALTH_CHOICES = (
    ("excellent", _("Excellent")),
    ("very_good", _("Very good")),
    ("good", _("Good")),
    ("fair", _("Fair")),
    ("poor", _("Poor")),
)

DYSLIPIDAEMIA_RX_CHOICES = (
    ("atorvastatin", "Atorvastatin"),
    ("rosuvastatin", "Rosuvastatin"),
    (OTHER, _("Other, specify below ...")),
    (NOT_APPLICABLE, _("Not applicable")),
)

ENDPOINT_CHOICES = (
    (YES, _(YES)),
    (PENDING, _("No. A repeat FBG will be scheduled")),
    (NO, _(NO)),
    (NOT_APPLICABLE, _("Not applicable")),
)

FEELING_DURATION_CHOICES = (
    (ALL_OF_THE_TIME, _("All of the time")),
    (MOST_OF_THE_TIME, _("Most of the time")),
    (GOOD_BIT_OF_THE_TIME, _("A good bit of the time")),
    (SOME_OF_THE_TIME, _("Some of the time")),
    (LITTLE_OF_THE_TIME, _("A little of the time")),
    (NONE_OF_THE_TIME, _("None of the time")),
)


FETAL_OUTCOMES = (
    (LIVE_AT_TERM, _("Live birth at term")),
    (LIVE_PRETERM, _("Live birth preterm")),
    ("stillbirth", _("Stillbirth")),
    ("miscarriage", _("Miscarriage")),
)

FOLLOWUP_REASONS = (
    (APPT, _("Study appointment")),
    (APPT_OTHER, _("Other routine appointment")),
    (UNSCHEDULED, _("Unschedule visit")),
    (OTHER, _("Other reason, specify below ...")),
)

GRADE34_CHOICES = (
    (GRADE3, _("Grade %(num)s") % {"num": 3}),
    (GRADE4, _("Grade %(num)s") % {"num": 4}),
    (NOT_APPLICABLE, _("Not applicable")),
)

HEALTH_LIMITED_CHOICES = (
    ("limited_a_lot", _("YES, limited a lot")),
    ("limited_a_little", _("YES, limited a little")),
    ("not_limited_at_all", _("NO, not at all limited")),
)

INFO_SOURCE = (
    ("hospital_notes", _("Hospital notes")),
    ("outpatient_cards", _("Outpatient cards")),
    ("patient", _("Patient")),
    ("collateral_history", _("Collateral History from relative/guardian")),
    (NOT_APPLICABLE, _("Not applicable (if missed)")),
    (OTHER, _("Other")),
)

DELIVERY_INFORMANT_RELATIONSHIP = list(INFORMANT_RELATIONSHIP)
DELIVERY_INFORMANT_RELATIONSHIP.extend(
    [
        (HOSPITAL_CLINIC, _("Hospital / Clinic records")),
        (NOT_APPLICABLE, _("Not applicable")),
    ]
)
DELIVERY_INFORMANT_RELATIONSHIP = tuple(DELIVERY_INFORMANT_RELATIONSHIP)

INTERFERENCE_DURATION_CHOICES = (
    (ALL_OF_THE_TIME, _("All of the time")),
    (MOST_OF_THE_TIME, _("Most of the time")),
    (SOME_OF_THE_TIME, _("Some of the time")),
    (LITTLE_OF_THE_TIME, _("A little of the time")),
    (NONE_OF_THE_TIME, _("None of the time")),
)

FUNDOSCOPY_CHOICES = (
    ("no_retinopathy", _("No retinopathy")),
    ("background_retinopathy", _("Background retinopathy")),
    ("pre_proliferative_retinopathy", _("Pre-proliferative retinopathy")),
    ("proliferative_retinopathy", _("Proliferative retinopathy")),
    ("maculopathy", _("Maculopathy")),
    (NO_EXAM, _("Exam not performed")),
)

MATERNAL_OUTCOMES = (
    (NO_COMPLICATIONS, _("No complications")),
    ("complications_full_recovery", _("Complications with full recovery")),
    ("complications_ongoing_recovery", _("Complications with ongoing recovery")),
    (DEAD, _("Maternal mortality")),
    (NOT_APPLICABLE, _("Not applicable")),
)

PAYEE_CHOICES = (
    ("own_cash", _("Own cash")),
    ("insurance", _("Insurance")),
    ("relative", _("Relative of others paying")),
    ("free", _("Free drugs from the pharmacy")),
    (NOT_APPLICABLE, _("Not applicable")),
)

PRESENT_ABSENT_NOEXAM = (
    (PRESENT, _("Present")),
    (ABSENT, _("Absent")),
    (NO_EXAM, _("Exam not performed")),
)

PRESENT_ABSENT_NOEXAM_NDS = (
    (PRESENT, _("Present")),
    (PRESENT_WITH_REINFORCEMENT, _("Present with reinforcement")),
    (ABSENT, _("Absent")),
    (NO_EXAM, _("Exam not performed")),
)
# 0 = Present   1 = Present with reinforcement   2 = Absent

REPORT_STATUS = (
    (NEW, _("New")),
    (OPEN, _("Open. Some information is still pending.")),
    (CLOSED, _("Closed. This report is complete")),
)


TRANSPORT_CHOICES = (
    ("bus", _("Bus")),
    ("train", _("Train")),
    ("ambulance", _("Ambulance")),
    ("private_taxi", _("Private taxi")),
    ("own_bicycle", _("Own bicycle")),
    ("hired_motorbike", _("Hired motorbike")),
    ("own_car", _("Own car")),
    ("own_motorbike", _("Own motorbike")),
    ("hired_bicycle", _("Hired bicycle")),
    ("foot", _("Foot")),
    (OTHER, _("Other, specify")),
)


VISIT_UNSCHEDULED_REASON = (
    ("patient_unwell_outpatient", _("Patient unwell (outpatient)")),
    ("patient_hospitalised", _("Patient hospitalised")),
    ("routine_non_study", _("Routine appointment (non-study)")),
    ("recurrence_symptoms", _("Recurrence of symptoms")),
    (OTHER, _("Other")),
    (NOT_APPLICABLE, _("Not applicable")),
)

VISIT_REASON = (
    (SCHEDULED, _("Scheduled visit")),
    (UNSCHEDULED, _("Unscheduled visit")),
    (MISSED_VISIT, _("Missed visit")),
)

WEIGHT_DETERMINATION = (("estimated", _("Estimated")), ("measured", _("Measured")))

WORK_PAIN_INTERFERENCE_CHOICES = (
    ("not_at_all", _("Not at all")),
    ("a_little_bit", _("A little bit")),
    ("moderately", _("Moderately")),
    ("quite_a-bit", _("Quite a bit")),
    ("extremely", _("Extremely")),
)

YES_NO_NO_EXAM = (
    (YES, _(YES)),
    (NO, _(NO)),
    (NO_EXAM, _("Exam not performed")),
)



from django.test import TestCase
from edc_action_item.models import ActionItem
from edc_constants.constants import FEMALE, NEW, PATIENT, YES
from edc_offstudy.constants import END_OF_STUDY_ACTION
from edc_pharmacy.models import Medication
from edc_transfer.constants import SUBJECT_TRANSFER_ACTION, TRANSFERRED
from edc_utils import get_utcnow
from edc_visit_schedule.constants import OFFSCHEDULE_ACTION

from meta_lists.models import OffstudyReasons, TransferReasons
from meta_pharmacy.constants import METFORMIN
from meta_prn.action_items import OffscheduleAction, SubjectTransferAction
from meta_prn.constants import OFFSTUDY_MEDICATION_ACTION
from meta_prn.models import EndOfStudy, OffSchedule, OffStudyMedication, SubjectTransfer
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin


class TestEosEvents(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(gender=FEMALE)
        self.subject_consent = self.get_subject_consent(self.subject_screening)
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )
        self.data = dict(
            subject_visit=self.subject_visit.pk,
            report_datetime=self.subject_visit.report_datetime,
        )

    def test_transfer_to_offschedule_in_order(self):
        SubjectTransferAction(
            subject_identifier=self.subject_consent.subject_identifier,
            skip_get_current_site=True,
            site_id=self.subject_consent.site_id,
        )
        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [SUBJECT_TRANSFER_ACTION])

        # add a subject transfer object, which triggers next action item
        transfer_reason = TransferReasons.objects.get(name="moved")
        subject_transfer = SubjectTransfer.objects.create(
            subject_identifier=self.subject_consent.subject_identifier,
            initiated_by="patient",
            may_return=YES,
            may_contact=YES,
        )
        subject_transfer.transfer_reason.add(transfer_reason)

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [OFFSCHEDULE_ACTION, OFFSTUDY_MEDICATION_ACTION])

        OffSchedule.objects.create(subject_identifier=self.subject_consent.subject_identifier)

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [OFFSTUDY_MEDICATION_ACTION])

        offstudy_rx = OffStudyMedication.objects.create(
            subject_identifier=self.subject_consent.subject_identifier,
            stop_date=get_utcnow().date(),
            last_dose_date=get_utcnow().date(),
            reason=PATIENT,
        )
        offstudy_rx.medications.add(Medication.objects.get(name=METFORMIN))

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [END_OF_STUDY_ACTION])

        EndOfStudy.objects.create(
            subject_identifier=self.subject_consent.subject_identifier,
            last_seen_date=get_utcnow().date(),
            offstudy_reason=OffstudyReasons.objects.get(name=TRANSFERRED),
        )

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [])

    def test_transfer_to_offschedule_raises(self):

        OffscheduleAction(
            subject_identifier=self.subject_consent.subject_identifier,
            skip_get_current_site=True,
            site_id=self.subject_consent.site_id,
        )

        OffSchedule.objects.create(subject_identifier=self.subject_consent.subject_identifier)

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [OFFSTUDY_MEDICATION_ACTION])

        offstudy_rx = OffStudyMedication.objects.create(
            subject_identifier=self.subject_consent.subject_identifier,
            stop_date=get_utcnow().date(),
            last_dose_date=get_utcnow().date(),
            reason=PATIENT,
        )
        offstudy_rx.medications.add(Medication.objects.get(name=METFORMIN))

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [END_OF_STUDY_ACTION])

        EndOfStudy.objects.create(
            subject_identifier=self.subject_consent.subject_identifier,
            last_seen_date=get_utcnow().date(),
            offstudy_reason=OffstudyReasons.objects.get(name=TRANSFERRED),
        )

        action_types = [
            obj.action_type.name
            for obj in ActionItem.objects.filter(status=NEW).order_by("action_type__name")
        ]
        self.assertEqual(action_types, [])



# Generated by Django 2.2.7 on 2019-11-19 20:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0015_auto_20191107_2249"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="fasting_glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt_two_hr_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
    ]



import os

from celery import Celery

# from kombu import Queue

# load settings
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "meta_edc.settings.debug")

# config celery
app = Celery("meta_edc")
app.config_from_object("django.conf:settings", namespace="CELERY")
app.autodiscover_tasks()

# app.conf.task_default_queue = "workuatq1"
# app.conf.task_queues = (Queue("workuatq1", routing_key="task.#"),)
# app.conf.task_default_exchange = "workuatq1"
# app.conf.task_default_exchange_type = "topic"
# app.conf.task_default_routing_key = "task.workuatq1"


@app.task(bind=True)
def debug_task(self):
    print("Request: {0!r}".format(self.request))



# Generated by Django 3.2.13 on 2022-09-28 01:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0153_auto_20220928_0250"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalstudymedication",
            name="next_dosage_guideline",
        ),
        migrations.RemoveField(
            model_name="historicalstudymedication",
            name="next_formulation",
        ),
        migrations.RemoveField(
            model_name="historicalstudymedication",
            name="order_or_update_next",
        ),
        migrations.RemoveField(
            model_name="studymedication",
            name="next_dosage_guideline",
        ),
        migrations.RemoveField(
            model_name="studymedication",
            name="next_formulation",
        ),
        migrations.RemoveField(
            model_name="studymedication",
            name="order_or_update_next",
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-13 04:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0169_alter_dmreferralfollowup_options_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="glucose",
            name="repeat_fbg",
        ),
        migrations.RemoveField(
            model_name="historicalglucose",
            name="repeat_fbg",
        ),
        migrations.AlterField(
            model_name="glucose",
            name="endpoint_today",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "No. A repeat FBG will be scheduled"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Answer yes or no if both the FBG and OGTT are available",
                max_length=25,
                verbose_name="Has the participant reached a study endpoint today?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="endpoint_today",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "No. A repeat FBG will be scheduled"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Answer yes or no if both the FBG and OGTT are available",
                max_length=25,
                verbose_name="Has the participant reached a study endpoint today?",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-08-12 00:35

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0078_auto_20210810_0857"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="haemoglobin_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="haemoglobin_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="hct_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="hct_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="platelets_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="platelets_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="rbc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="rbc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="wbc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="wbc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="albumin_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="albumin_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="alp_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="alp_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="alt_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="alt_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="amylase_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="amylase_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="ast_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="ast_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="ggt_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="ggt_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="hdl_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="hdl_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="ldl_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="ldl_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="trig_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="trig_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="creatinine_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="creatinine_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="urea_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="urea_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="uric_acid_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="uric_acid_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="haemoglobin_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="haemoglobin_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="hct_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="hct_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="platelets_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="platelets_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="rbc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="rbc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="wbc_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="wbc_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="albumin_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="albumin_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="alp_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="alp_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="alt_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="alt_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="amylase_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="amylase_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="ast_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="ast_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="ggt_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="ggt_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="hdl_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="hdl_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="ldl_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="ldl_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="trig_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="trig_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="creatinine_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="creatinine_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="urea_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="urea_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_grade",
            field=models.IntegerField(
                blank=True,
                choices=[
                    (0, "Not graded"),
                    (1, "Grade 1"),
                    (2, "Grade 2"),
                    (3, "Grade 3"),
                    (4, "Grade 4"),
                    (5, "Grade 5"),
                ],
                null=True,
                verbose_name="Grade",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_grade_description",
            field=models.CharField(
                blank=True, max_length=250, null=True, verbose_name="Grade description"
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-10 19:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0024_auto_20210710_1929"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="staying_nearby",
            new_name="staying_nearby_6",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="staying_nearby",
            new_name="staying_nearby_6",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="staying_nearby",
            new_name="staying_nearby_6",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="staying_nearby",
            new_name="staying_nearby_6",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="staying_nearby",
            new_name="staying_nearby_6",
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="has_dm",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to have diabetes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="on_dm_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to be taking anti-diabetic medications?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="has_dm",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to have diabetes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="on_dm_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to be taking anti-diabetic medications?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="has_dm",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to have diabetes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="on_dm_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to be taking anti-diabetic medications?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="has_dm",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to have diabetes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="on_dm_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to be taking anti-diabetic medications?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="has_dm",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to have diabetes?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="on_dm_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient known to be taking anti-diabetic medications?",
            ),
        ),
    ]



from django_db_views.db_view import DBView
from edc_qareports.model_mixins import (
    OnStudyMissingValuesModelMixin,
    QaReportModelMixin,
    qa_reports_permissions,
)

from .view_definition import get_view_definition


class OnStudyMissingValues(OnStudyMissingValuesModelMixin, QaReportModelMixin, DBView):

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "onstudy_missing_values_view"
        verbose_name = "Missing values for on-study patient"
        verbose_name_plural = "Missing values for on-study patients"
        default_permissions = qa_reports_permissions



import sys

from django.core.management import BaseCommand
from edc_consent.utils import get_consent_model_cls
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.prescribe import create_prescription
from edc_visit_schedule.constants import DAY1
from tqdm import tqdm

from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN
from meta_subject.models import SubjectVisit


def create_missing_rx():
    n = 0
    sys.stdout.write("\nCreate missing prescriptions...\n")
    cnt = SubjectVisit.objects.filter(visit_code=DAY1, visit_code_sequence=0).count()
    for subject_visit in tqdm(
        SubjectVisit.objects.filter(visit_code=DAY1, visit_code_sequence=0), total=cnt
    ):
        subject_consent = get_consent_model_cls().objects.get(
            subject_identifier=subject_visit.subject_identifier,
        )
        try:
            create_prescription(
                subject_identifier=subject_consent.subject_identifier,
                report_datetime=subject_consent.consent_datetime,
                randomizer_name=get_meta_version(),
                medication_names=[METFORMIN],
                site=subject_consent.site,
            )
        except PrescriptionAlreadyExists:
            pass
        else:
            n += 1

    sys.stdout.write(f"\nCreated {n}/{cnt}. Done.\n")


class Command(BaseCommand):
    def handle(self, *args, **options):
        create_missing_rx()



# Generated by Django 3.2.6 on 2021-09-23 22:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0088_auto_20210924_0027"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eq5d3l",
            name="anxiety_depression",
            field=models.CharField(
                choices=[
                    ("not_anxious_depressed", "I am not anxious or depressed"),
                    (
                        "moderately_anxious_depressed",
                        "I am moderately anxious or depressed",
                    ),
                    (
                        "extremely_anxious_depressed",
                        "I am extremely anxious or depressed",
                    ),
                ],
                max_length=45,
                verbose_name="Anxiety / Depression",
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="health_today_score_slider",
            field=models.CharField(
                max_length=3, verbose_name="Visual score for how your health is TODAY"
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="pain_discomfort",
            field=models.CharField(
                choices=[
                    ("no_pain_discomfort", "I have no pain or discomfort"),
                    ("moderate_pain_discomfort", "I have moderate pain or discomfort"),
                    ("extreme_pain_discomfort", "I have extreme pain or discomfort"),
                ],
                max_length=45,
                verbose_name="Pain / Discomfort",
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="self_care",
            field=models.CharField(
                choices=[
                    ("no_problems_with_self_care", "I have no problems with self-care"),
                    (
                        "problems_washing_dressing_myself",
                        "I have some problems washing or dressing myself",
                    ),
                    (
                        "unable_to_wash_dress_myself",
                        "I am unable to wash or dress myself",
                    ),
                ],
                max_length=45,
                verbose_name="Self-care",
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="usual_activities",
            field=models.CharField(
                choices=[
                    (
                        "no_problems_performing_usual_activities",
                        "I have no problems with performing my usual activities",
                    ),
                    (
                        "some_problems_performing_usual_activities",
                        "I have some problems with performing my usual activities",
                    ),
                    (
                        "unable_to_perform_usual_activities",
                        "I am unable to perform my usual activities",
                    ),
                ],
                help_text="Example. work, study, housework, family or leisure activities",
                max_length=45,
                verbose_name="Usual activities",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="anxiety_depression",
            field=models.CharField(
                choices=[
                    ("not_anxious_depressed", "I am not anxious or depressed"),
                    (
                        "moderately_anxious_depressed",
                        "I am moderately anxious or depressed",
                    ),
                    (
                        "extremely_anxious_depressed",
                        "I am extremely anxious or depressed",
                    ),
                ],
                max_length=45,
                verbose_name="Anxiety / Depression",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="health_today_score_slider",
            field=models.CharField(
                max_length=3, verbose_name="Visual score for how your health is TODAY"
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="pain_discomfort",
            field=models.CharField(
                choices=[
                    ("no_pain_discomfort", "I have no pain or discomfort"),
                    ("moderate_pain_discomfort", "I have moderate pain or discomfort"),
                    ("extreme_pain_discomfort", "I have extreme pain or discomfort"),
                ],
                max_length=45,
                verbose_name="Pain / Discomfort",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="self_care",
            field=models.CharField(
                choices=[
                    ("no_problems_with_self_care", "I have no problems with self-care"),
                    (
                        "problems_washing_dressing_myself",
                        "I have some problems washing or dressing myself",
                    ),
                    (
                        "unable_to_wash_dress_myself",
                        "I am unable to wash or dress myself",
                    ),
                ],
                max_length=45,
                verbose_name="Self-care",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="usual_activities",
            field=models.CharField(
                choices=[
                    (
                        "no_problems_performing_usual_activities",
                        "I have no problems with performing my usual activities",
                    ),
                    (
                        "some_problems_performing_usual_activities",
                        "I have some problems with performing my usual activities",
                    ),
                    (
                        "unable_to_perform_usual_activities",
                        "I am unable to perform my usual activities",
                    ),
                ],
                help_text="Example. work, study, housework, family or leisure activities",
                max_length=45,
                verbose_name="Usual activities",
            ),
        ),
    ]



from django.db import models
from edc_constants.constants import NOT_APPLICABLE
from edc_lab.choices import SERUM_CREATININE_UNITS_NA


class CreatinineModelFieldsMixin(models.Model):
    creatinine_value = models.DecimalField(
        verbose_name="Creatinine level",
        max_digits=8,
        decimal_places=2,
        null=True,
        blank=True,
    )

    creatinine_units = models.CharField(
        verbose_name="Units (creatinine)",
        max_length=15,
        choices=SERUM_CREATININE_UNITS_NA,
        default=NOT_APPLICABLE,
    )

    class Meta:
        abstract = True



from edc_constants.constants import (
    CLINIC,
    DEAD,
    DELIVERY,
    DIABETES,
    NONE,
    OTHER,
    PREGNANCY,
    TOXICITY,
    UNKNOWN,
)
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_offstudy.constants import COMPLETED_FOLLOWUP, WITHDRAWAL
from edc_transfer.constants import TRANSFERRED

from meta_prn.constants import (
    CLINICAL_WITHDRAWAL,
    COMPLETED_FOLLOWUP_48,
    LATE_EXCLUSION,
)

list_data = {
    "meta_lists.dmmedications": [
        ("metformin", "Metformin"),
        ("glibenclamide", "Glibenclamide"),
        ("gliclazide", "Gliclazide"),
        ("pioglitazone", "Pioglitazone"),
        ("sitagliptin", "Sitagliptin"),
        ("Vildagliptin", "vildagliptin"),
        ("dapagliflozin", "Dapagliflozin"),
        ("epagliflozin", "Epagliflozin"),
        ("insulin", "Insulin"),
        (OTHER, "Other, please specify"),
    ],
    "meta_lists.missedreferralreasons": [
        ("distance", "Distance to the clinic"),
        ("travel_costs", "Cost of travel to the clinic"),
        ("unable_to_pay_meds", "Cannot pay for the medication"),
        ("dont_want_more_meds", "I do not want to take more medication"),
        ("want_diet_lifestyle_only", "I planned on doing diet and lifestyle changes only"),
        ("traditional_healer", "Visited a traditional healer"),
        ("not_sick", "I thought I was well and had no diabetes"),
        ("dont_like_clinic", "Do not like the service at the clinic"),
        (OTHER, "Other, please specify"),
    ],
    "meta_lists.dmtreatments": [
        ("diet_lifestyle", "Diet and lifestyle"),
        ("drugs", "Oral drugs"),
        ("insulin", "Insulin injections"),
    ],
    "meta_lists.complications": [
        ("foot", "Foot check"),
        ("eye_disease", "Diabetic eye disease"),
        ("kidney_disease", "Kidney disease"),
        ("heart_disease", "Heart disease"),
    ],
    "meta_lists.investigations": [
        ("blood_glucose", "Blood sugar test"),
        ("other_tests", "Other blood tests"),
        ("physical_exam", "Physical examination"),
        (OTHER, "Other, please specify"),
    ],
    "meta_lists.healthcareworkers": [
        ("nurse", "Nurse"),
        ("Doctor", "Doctor"),
        ("community_hcw", "Community healthcare worker"),
        (OTHER, "Other, please specify"),
    ],
    "meta_lists.abnormalfootappearanceobservations": [
        ("deformities", "Deformities"),
        ("dry_skin_callus", "Dry skin, callus"),
        ("infection", "Infection"),
        ("fissure", "Fissure"),
        (OTHER, "Other abnormality, please specify"),
    ],
    "meta_lists.nonadherencereasons": [
        ("forget_to_take", "I sometimes forget to take my pills"),
        ("dont_like_taking", "I don't like taking my pills"),
        ("make_me_ill", "My pills sometimes make me feel sick"),
        ("misplaced_pills", "I sometimes misplace my pills"),
        ("dont_believe_pills_help", "I don't believe my pills are helping me"),
        ("dont_believe_pills_needed", "I don't believe I need to take my pills"),
        ("not_feeling_well", "I have not been feeling well"),
        (OTHER, "Other, please specify ..."),
    ],
    "meta_lists.hypertensionmedications": [
        ("amlodipine", "Amlodipine"),
        ("atenolol", "Atenolol"),
        ("bendroflumethiazide", "Bendroflumethiazide"),
        ("bisoprolol", "Bisoprolol"),
        ("captopril", "Captopril"),
        ("diltiazem", "Diltiazem"),
        ("enalapril", "Enalapril"),
        ("eplerenone", "Eplerenone"),
        ("furosemide", "Furosemide"),
        ("hydralazine", "Hydralazine"),
        ("hydrochlothiazide", "Hydrochlothiazide"),
        ("indapamide", "Indapamide"),
        ("losartan", "Losartan"),
        ("metoprolol", "Metoprolol"),
        ("nifedipine", "Nifedipine"),
        ("perindopril", "Perindopril"),
        ("propanolol", "Propanolol"),
        ("spironolactone", "Spironolactone"),
        ("telmisartan", "Telmisartan"),
        ("torsemide", "Torsemide"),
        ("verapamil", "Verapamil"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.diabetessymptoms": [
        ("frequent_urination", "Wanting to urinate more often than usual"),
        ("excessive_thirst", "Wanting to drink water more than usual"),
        ("excessive_eating", "Wanting to eat food more than usual"),
        (OTHER, "Other, specify"),
        (NONE, "No symptoms to report"),
    ],
    "meta_lists.oiprophylaxis": [
        ("tmp_smx", "TMP/SMX"),
        ("fluconazole", "Fluconazole"),
        ("isoniazid", "Isoniazid"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.symptoms": [
        (NONE, "--No symptoms to report"),
        ("abdominal_pain_general", "Abdominal pain (General)"),
        ("abdominal_pain_right_upper_quad", "Abdominal pain (Right upper quadrant)"),
        ("blurred_vision", "Blurred vision"),
        ("diarrhoea", "Diarrhoea"),
        ("dizziness", "Dizziness"),
        ("fatigue", "Fatigue"),
        ("flatulence", "Flatulence (gas)"),
        ("headaches", "Headaches"),
        ("increased_appetite", "Increased appetite"),
        ("joint_pain", "Joint pain"),
        ("loss_of_appetite", "Loss of appetite"),
        ("muscle_cramping", "Muscle cramping"),
        ("muscle_pain", "Muscle pain"),
        ("nausea", "Nausea"),
        ("pain_feet_or_lower_limbs", "Pain in feet/lower limbs"),
        ("pounding_heartbeat", "Fast or pounding heartbeat"),
        ("rash", "Rash"),
        ("shakiness", "Shakiness"),
        ("shallow_breathing", "Fast or shallow breathing"),
        ("skin_itching", "Skin itching"),
        ("sweating", "Sweating"),
        ("swelling_feet_or_lower_limbs", "Swelling of feet/lower limbs"),
        ("unusual_sleepiness", "Unusual sleepiness"),
        ("vomiting", "Vomiting"),
        ("weakness", "Weakness"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.baselinesymptoms": [
        ("abdominal_pain_general", "Abdominal pain (General)"),
        ("cough", "Cough"),
        ("diarrhoea", "Diarrhoea"),
        ("dizziness", "Dizziness"),
        ("fatigue", "Fatigue"),
        ("headaches", "Headaches"),
        ("loss_of_appetite", "Loss of appetite"),
        ("loss_of_weight", "Loss of weight"),
        ("muscle_cramping", "Muscle cramping"),
        ("shakiness", "Shakiness"),
        ("shallow_breathing", "Fast or shallow breathing"),
        ("skin_infection", "Skin infection"),
        ("sweating", "Sweating"),
        ("unusual_sleepiness", "Unusual sleepiness"),
        ("vomiting", "Vomiting"),
        ("weakness", "Weakness"),
        (NONE, "No symptoms to report"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.arvregimens": [
        ("ABC_3TC_ATV_r", "ABC + 3TC + ATV/r"),
        ("ABC_3TC_LPV_r", "ABC + 3TC + LPV/r"),
        ("AZT_3TC_ATV_r", "AZT + 3TC + ATV/r"),
        ("AZT_3TC_EFV", "AZT + 3TC + EFV"),
        ("AZT_3TC_LPV_r", "AZT + 3TC + LPV/r"),
        ("AZT_3TC_NVP", "AZT + 3TC + NVP"),
        ("D4T_3TC_NVP", "D4T + 3TC + NVP"),
        ("DTG_ABC/3TC_ATV_r", "DTG + (ABC/3TC) + ATV/r"),
        ("TDF_3TC_ATV_r", "TDF + 3TC + ATV/r"),
        ("TDF_3TC_DTG", "TDF + 3TC + DTG"),
        ("TDF_3TC_EFV", "TDF + 3TC + EFV"),
        ("TDF_3TC_LPV_r", "TDF + 3TC + LPV/r"),
        ("TDF_FTC_ATV_r", "TDF + FTC + ATV/r"),
        ("TDF_FTC_EFV", "TDF + FTC + EFV"),
        ("TDF_FTC_LPV_r", "TDF + FTC + LPV/r"),
        ("ZDV_3TC_EFV", "ZDV + 3TC + EFV"),
        ("ZDV_3TC_NVP", "ZDV + 3TC + NVP"),
        ("ZDV_LPV_NVP", "ZDV + LPV + NVP"),
        (UNKNOWN, "Unknown"),
        (OTHER, "Other, specify ..."),
    ],
    "meta_lists.subjectvisitmissedreasons": [
        ("forgot", "Forgot / Can't remember being told about appointment"),
        ("family_emergency", "Family emergency (e.g. funeral) and was away"),
        ("travelling", "Away travelling/visiting"),
        ("working_schooling", "Away working/schooling"),
        ("too_sick", "Too sick or weak to come to the centre"),
        ("lack_of_transport", "Transportation difficulty"),
        (OTHER, "Other reason (specify below)"),
    ],
    "meta_lists.offstudyreasons": [
        (COMPLETED_FOLLOWUP, "Patient completed 36 months of follow-up"),
        (COMPLETED_FOLLOWUP_48, "Patient completed 48 months of follow-up"),
        (DELIVERY, "Delivered / Completed followup from pregnancy"),
        (PREGNANCY, "Pregnancy, declined further followup"),
        (DIABETES, "Patient developed diabetes"),
        (CLINICAL_WITHDRAWAL, "Patient is withdrawn on CLINICAL grounds ..."),
        ("clinical_endpoint", "Patient reached a clinical endpoint"),  # see limit_choices_to
        (
            TOXICITY,
            "Patient experienced an unacceptable toxicity",
        ),
        (
            "intercurrent_illness",
            "Intercurrent illness which prevents further treatment",  # see limit_choices_to
        ),
        (LOST_TO_FOLLOWUP, "Patient lost to follow-up"),
        (DEAD, "Patient reported/known to have died"),
        (WITHDRAWAL, "Patient withdrew consent to participate further"),
        (
            LATE_EXCLUSION,
            (
                "Patient fulfilled late exclusion criteria (due to abnormal "
                "blood values or raised blood pressure at enrolment"
            ),
        ),
        (TRANSFERRED, "Patient has been transferred to another health centre"),
        (OTHER, "Other reason (specify below)"),
    ],
    "meta_lists.transferreasons": [
        ("moved", "Moved away from the area"),
        ("unhappy_with_care", "Was unhappy with the care they are receiving"),
        ("need_specialized_treatment", "Needed more specialist treatment"),
        ("drug_supply", "Due to drug supply "),
        ("stigma", "Concerned about stigma"),
        (OTHER, "Other reason (specify below)"),
    ],
    "edc_facility.healthfacilitytypes": [
        (CLINIC, "META Study Clinic"),
    ],
}



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
        select s.subject_identifier, s.sid, s.dispensed_sid, arm_match,
        s.report_datetime, r.allocated_datetime,
        s.site_id, s.user_created, s.user_modified, s.modified, s.id as original_id
        from meta_pharmacy_substitutions as s left join meta_rando_randomizationlist as r
        on r.subject_identifier=s.subject_identifier
        order by s.subject_identifier
        """  # noqa
    sql_view = SqlViewGenerator(
        report_model="meta_reports.imp_subjectitutions_view",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from edc_constants.constants import OK

METFORMIN = "metformin"
REVIEW = "REVIEW"
MISSING_SUBJECT_IDENTIFIER = "MISSING_SUBJECT_IDENTIFIER"

SUBSTITUTION_STATUS = (
    (OK, "ok"),
    (MISSING_SUBJECT_IDENTIFIER, "Missing Subject ID"),
    (REVIEW, "review"),
)



from datetime import datetime
from zoneinfo import ZoneInfo

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_appointment.constants import NEW_APPT
from edc_constants.constants import YES
from edc_lab_panel.panels import hba1c_panel, insulin_panel
from edc_metadata.metadata_rules import PersistantSingletonMixin
from edc_sites.site import sites
from edc_visit_schedule.constants import (
    DAY1,
    MONTH1,
    MONTH3,
    MONTH6,
    MONTH12,
    MONTH18,
    MONTH24,
    MONTH30,
    MONTH36,
    MONTH48,
    WEEK2,
)
from edc_visit_schedule.utils import is_baseline

from meta_visit_schedule.constants import MONTH39, MONTH42, MONTH45


def hba1c_crf_required_at_baseline(visit):
    model_name = "meta_subject.bloodresultshba1c"
    screening_model_cls = django_apps.get_model("meta_screening.subjectscreening")
    bloodresults_model_cls = django_apps.get_model(model_name)
    required = False
    if is_baseline(instance=visit):
        try:
            screening_model_cls.objects.get(
                subject_identifier=visit.subject_identifier,
                hba1c_value__isnull=True,
            )
        except ObjectDoesNotExist:
            pass
        else:
            try:
                obj = bloodresults_model_cls.objects.get(
                    subject_visit__visit_code=DAY1,
                    subject_visit__visit_code_sequence=0,
                )
            except ObjectDoesNotExist:
                required = True
            else:
                if not obj.hba1c_value:
                    required = True
    return required


def hba1c_requisition_required_at_baseline(visit):
    screening_model_cls = django_apps.get_model("meta_screening.subjectscreening")
    required = False
    if is_baseline(instance=visit):
        try:
            screening_model_cls.objects.get(
                subject_identifier=visit.subject_identifier,
                hba1c_value__isnull=False,
            )
        except ObjectDoesNotExist:
            required = True
    return required


class Predicates(PersistantSingletonMixin):
    app_label = "meta_subject"

    @staticmethod
    def next_appt_required(visit, **kwargs):
        if visit.appointment.next and visit.appointment.next.appt_status == NEW_APPT:
            return True
        return False

    @staticmethod
    def glucose_required(visit, **kwargs):
        if visit.visit_code in [MONTH12, MONTH24, MONTH36, MONTH48]:
            return True
        return False

    @staticmethod
    def glucose_fbg_required(visit, **kwargs):
        if visit.report_datetime >= datetime(2024, 3, 4, tzinfo=ZoneInfo("UTC")):
            if visit.visit_code in [MONTH6, MONTH18, MONTH30, MONTH42]:
                return True
        return False

    @staticmethod
    def pregnancy_notification_exists(visit, **kwargs):
        model_cls = django_apps.get_model("meta_prn.pregnancynotification")
        try:
            model_cls.objects.get(subject_identifier=visit.subject_identifier, delivered=False)
        except ObjectDoesNotExist:
            required = False
        else:
            required = True
        return required

    @staticmethod
    def hba1c_crf_required(visit, **kwargs) -> bool:
        """Require at baseline visit if not recorded on the
        screening form.
        """
        required = hba1c_crf_required_at_baseline(visit)
        if (
            not required
            and visit.appointment.visit_code != DAY1
            and visit.appointment.visit_code_sequence == 0
            and visit.appointment.visit_code
            in visit.schedule.crf_required_at("meta_subject.bloodresultshba1c")
        ):
            required = True
        return required

    @staticmethod
    def hba1c_requisition_required(visit, **kwargs) -> bool:
        """Require at baseline visit if not recorded on the
        screening form.
        """
        required = hba1c_requisition_required_at_baseline(visit)
        if (
            not required
            and visit.appointment.visit_code != DAY1
            and visit.appointment.visit_code_sequence == 0
            and visit.appointment.visit_code
            in visit.schedule.requisition_required_at(hba1c_panel)
        ):
            required = True
        return required

    @staticmethod
    def insulin_crf_required(visit, **kwargs) -> bool:
        """Require at baseline visit"""
        required = False
        if (
            visit.site.id
            in [
                sites.get_by_attr("name", "mwananyamala").site_id,
                sites.get_by_attr("name", "temeke").site_id,
            ]
            and visit.appointment.visit_code
            in visit.schedule.crf_required_at("meta_subject.bloodresultsins")
            and visit.appointment.visit_code_sequence == 0
        ):
            required = True
        return required

    @staticmethod
    def insulin_requisition_required(visit, **kwargs) -> bool:
        """Require at baseline visit"""
        required = False
        if (
            visit.site.id
            in [
                sites.get_by_attr("name", "mwananyamala").site_id,
                sites.get_by_attr("name", "temeke").site_id,
            ]
            and visit.appointment.visit_code
            in visit.schedule.requisition_required_at(insulin_panel)
            and visit.appointment.visit_code_sequence == 0
        ):
            required = True
        return required

    def health_economics_required(self, visit, **kwargs) -> bool:
        """Returns true if HE was not completed at week 2"""

        required = False
        if (
            visit.appointment.visit_code == WEEK2
            and visit.appointment.visit_code_sequence == 0
        ):
            required = True
        elif (
            visit.appointment.visit_code == MONTH1
            and visit.appointment.visit_code_sequence == 0
        ):
            model_cls = django_apps.get_model(f"{self.app_label}.healtheconomicssimple")
            try:
                model_cls.objects.get(
                    subject_visit__subject_identifier=visit.subject_identifier,
                    subject_visit__visit_code=WEEK2,
                    subject_visit__visit_code_sequence=0,
                )
            except ObjectDoesNotExist:
                required = True
        return required

    def health_economics_update_required(self, visit, **kwargs) -> bool:
        """Returns true if `healtheconomicsupdate` was not completed at
        month 3 or ever.

        `healtheconomicsupdate` is a singleton CRF.
        """
        required = False
        model_cls = django_apps.get_model(f"{self.app_label}.healtheconomicsupdate")
        try:
            obj = model_cls.objects.get(
                subject_visit__subject_identifier=visit.subject_identifier,
            )
        except ObjectDoesNotExist:
            obj = None
        if (
            not obj
            and visit.appointment.visit_code_sequence == 0
            and visit.appointment.visit_code not in [DAY1, WEEK2, MONTH1]
        ):
            required = True
        return required

    def mnsi_required(self, visit, **kwargs) -> bool:
        """Returns True if:
        - MNSI assessment not performed in the 1, 3 or 6M visits
        - MNSI assessment not performed in the 36, 39, 42, 45M visits
        - MNSI assessment not performed in the 48M visit
        """
        model_cls = django_apps.get_model(f"{self.app_label}.mnsi")
        required = True

        if self.offschedule_today(visit):
            return True
        elif visit.visit_code_sequence != 0:
            required = False
        elif visit.visit_code not in [
            MONTH1,
            MONTH3,
            MONTH6,
            MONTH36,
            MONTH39,
            MONTH42,
            MONTH45,
            MONTH48,
        ]:
            required = False
        elif visit.visit_code in [MONTH1, MONTH3, MONTH6]:
            objs = model_cls.objects.filter(
                subject_visit__subject_identifier=visit.subject_identifier,
                subject_visit__visit_code__in=[MONTH1, MONTH3, MONTH6],
                subject_visit__visit_code_sequence=0,
            )
            for obj in objs:
                if obj.mnsi_performed == YES:
                    required = False
                    break
        elif visit.visit_code in [MONTH36, MONTH39, MONTH42, MONTH45]:
            objs = model_cls.objects.filter(
                subject_visit__subject_identifier=visit.subject_identifier,
                subject_visit__visit_code__in=[MONTH36, MONTH39, MONTH42, MONTH45],
                subject_visit__visit_code_sequence=0,
            )
            for obj in objs:
                if obj.mnsi_performed == YES:
                    required = False
                    break
        return required

    def sf12_required(self, visit, **kwargs):
        model = f"{self.app_label}.sf12"
        if self.offschedule_today(visit):
            return True
        elif visit.visit_code_sequence == 0 and visit.visit_code in [MONTH36, MONTH48]:
            return True
        return self.persistant_singleton_required(
            visit, model=model, exclude_visit_codes=[DAY1]
        )

    def eq5d3l_required(self, visit, **kwargs):
        model = f"{self.app_label}.eq5d3l"
        if self.offschedule_today(visit):
            return True
        elif visit.visit_code_sequence == 0 and visit.visit_code in [MONTH36, MONTH48]:
            return True
        return self.persistant_singleton_required(
            visit, model=model, exclude_visit_codes=[DAY1]
        )

    def offschedule_today(self, visit):
        try:
            obj = django_apps.get_model(f"{self.app_label}.nextappointment").objects.get(
                subject_visit=visit
            )
        except ObjectDoesNotExist:
            pass
        else:
            if obj.offschedule_today == YES:
                return True
        return False



from datetime import datetime
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.core.exceptions import ValidationError
from django.test import TestCase, tag
from edc_appointment.exceptions import AppointmentWindowError
from edc_constants.constants import CLINIC
from edc_facility.models import HealthFacilityTypes
from edc_facility.utils import get_health_facility_model, get_health_facility_model_cls
from edc_metadata.metadata_handler import MetadataHandlerError
from edc_sites.utils import get_site_model_cls
from edc_visit_schedule.models import VisitSchedule

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_screening.tests.options import now
from meta_subject.models import NextAppointment


@time_machine.travel(datetime(2019, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
class TestNextAppointment(MetaTestCaseMixin, TestCase):

    def setUp(self):
        super().setUp()
        self.update_health_facility_model()
        self.subject_visit = self.get_subject_visit(appt_datetime=now)

    def update_health_facility_model(self):
        from edc_sites.site import sites as site_sites

        clinic = HealthFacilityTypes.objects.get(name=CLINIC)
        for site_obj in get_site_model_cls().objects.all():
            single_site = site_sites.get(site_obj.id)
            get_health_facility_model_cls().objects.create(
                name=single_site.name,
                title=single_site.title,
                health_facility_type=clinic,
                mon=True,
                tue=True,
                wed=True,
                thu=True,
                fri=True,
                sat=False,
                sun=False,
                site_id=site_obj.id,
            )

    @tag("6")
    def test_in_visit_crfs(self):
        dt = self.subject_visit.report_datetime.date() + relativedelta(days=10)
        for i in range(1, 7):
            dt = dt + relativedelta(days=1)
            if dt.weekday() < 6:
                break

        try:
            NextAppointment.objects.create(
                subject_visit=self.subject_visit,
                report_datetime=self.subject_visit.report_datetime,
                appt_date=dt,
                health_facility=get_health_facility_model_cls().objects.all()[0],
            )
        except MetadataHandlerError as e:
            self.fail(f"Unexpected MetadataHandlerError. Got {e}")

    @tag("6")
    def test_validates_clinic_day(self):
        dt = self.subject_visit.report_datetime.date() + relativedelta(days=10)
        for i in range(1, 7):
            dt = dt + relativedelta(days=1)
            if dt.weekday() > 5:
                break
        with self.assertRaises(ValidationError) as cm:
            NextAppointment.objects.create(
                subject_visit=self.subject_visit,
                report_datetime=self.subject_visit.report_datetime,
                appt_date=dt,
                health_facility=get_health_facility_model_cls().objects.all()[0],
            )
        self.assertIn("Expected Mon-Fri", str(cm.exception))

    @tag("6")
    def test_appt_date_on_report_date_raises(self):
        with self.assertRaises(ValidationError) as cm:
            NextAppointment.objects.create(
                subject_visit=self.subject_visit,
                report_datetime=self.subject_visit.report_datetime,
                appt_date=self.subject_visit.report_datetime.date(),
                health_facility=get_health_facility_model_cls().objects.all()[0],
            )
        self.assertIn(
            "Cannot be equal to the report datetime",
            str(cm.exception),
        )

    # @tag("6")
    # def test_next_appt_date_required(self):
    #     next_appt = self.subject_visit.appointment.next
    #     with self.assertRaises(NextAppointmentModelError) as cm:
    #         NextAppointment.objects.create(
    #             subject_visit=self.subject_visit,
    #             report_datetime=self.subject_visit.report_datetime,
    #             health_facility=get_health_facility_model_cls().objects.all()[0],
    #             visitschedule=VisitSchedule.objects.get(
    #                 visit_schedule_name=self.subject_visit.visit_schedule.name,
    #                 timepoint=next_appt.timepoint,
    #             ),
    #         )
    #     self.assertIn(
    #         "Appointment date or datetime is required",
    #         str(cm.exception),
    #     )

    @tag("6")
    def test_nextappt_appt_date_updates_nextappt_appt_datetime(self):
        next_appt = self.subject_visit.appointment.next
        original_next_appt_datetime = self.subject_visit.appointment.next.appt_datetime
        obj = NextAppointment.objects.create(
            appt_date=original_next_appt_datetime.date(),
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=next_appt.timepoint,
            ),
        )
        self.assertIsNotNone(obj.appt_datetime)

    # @tag("6")
    # def test_nextappt_appt_datetime_updates_nextappt_appt_date(self):
    #     next_appt = self.subject_visit.appointment.next
    #     original_next_appt_datetime = self.subject_visit.appointment.next.appt_datetime
    #     obj = NextAppointment.objects.create(
    #         appt_datetime=original_next_appt_datetime,
    #         subject_visit=self.subject_visit,
    #         report_datetime=self.subject_visit.report_datetime,
    #         health_facility=get_health_facility_model_cls().objects.all()[0],
    #         visitschedule=VisitSchedule.objects.get(
    #             visit_schedule_name=self.subject_visit.visit_schedule.name,
    #             timepoint=next_appt.timepoint,
    #         ),
    #     )
    #     self.assertIsNotNone(obj.appt_date)

    @tag("6")
    def test_next_appt_date_same_as_original_next_appt(self):
        get_health_facility_model()
        next_appt = self.subject_visit.appointment.next
        obj = NextAppointment.objects.create(
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            appt_date=next_appt.appt_datetime.date(),
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=next_appt.timepoint,
            ),
        )
        next_appt.refresh_from_db()
        self.assertEqual(next_appt.appt_datetime, obj.appt_datetime)

    @tag("6")
    def test_updates_next_appointment_datetime(self):
        next_appt = self.subject_visit.appointment.next
        original_next_appt_datetime = self.subject_visit.appointment.next.appt_datetime
        obj = NextAppointment.objects.create(
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            appt_date=original_next_appt_datetime.date() + relativedelta(days=1),
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=next_appt.timepoint,
            ),
        )
        next_appt.refresh_from_db()
        self.assertEqual(next_appt.appt_datetime, obj.appt_datetime)

    @tag("6")
    def test_raises_if_next_appointment_datetime_is_before_current(self):
        next_appt = self.subject_visit.appointment.next
        current_appt_datetime = self.subject_visit.appointment.appt_datetime
        self.assertRaises(
            ValidationError,
            NextAppointment.objects.create,
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            appt_date=current_appt_datetime.date(),
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=next_appt.timepoint,
            ),
        )

    @tag("6")
    def test_raises_on_appt_date_outside_of_window_for_selected_visit_code(self):
        next_appt = self.subject_visit.appointment.next
        bad_next_date = self.subject_visit.report_datetime.date() + relativedelta(days=1)
        self.assertRaises(
            AppointmentWindowError,
            NextAppointment.objects.create,
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            appt_date=bad_next_date,
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=next_appt.timepoint,
            ),
        )

    @tag("6")
    def test_next_is_interim_or_unscheduled(self):
        dt = self.subject_visit.report_datetime.date() + relativedelta(days=10)
        for i in range(1, 7):
            dt = dt + relativedelta(days=1)
            if dt.weekday() < 6:
                break
        NextAppointment.objects.create(
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            appt_date=dt,
            health_facility=get_health_facility_model_cls().objects.all()[0],
            visitschedule=VisitSchedule.objects.get(
                visit_schedule_name=self.subject_visit.visit_schedule.name,
                timepoint=self.subject_visit.appointment.timepoint,
            ),
        )



from django import forms
from edc_constants.constants import YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import PregnancyUpdate


class PregnancyUpdateFormValidator(CrfFormValidator):
    def clean(self):
        self.required_if(YES, field="contact", field_required="comment")


class PregnancyUpdateForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = PregnancyUpdateFormValidator

    class Meta:
        model = PregnancyUpdate
        fields = "__all__"



from django import forms
from edc_crf.modelform_mixins import InlineCrfModelFormMixin

from ..form_validators import BirthOutcomesFormValidator
from ..models import BirthOutcomes


class BirthOutcomesForm(InlineCrfModelFormMixin, forms.ModelForm):
    form_validator_cls = BirthOutcomesFormValidator

    class Meta:
        model = BirthOutcomes
        fields = "__all__"



# Generated by Django 4.0.5 on 2022-06-29 14:17

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_action_item.models.action_model_mixin
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("edc_action_item", "0028_auto_20210203_0706"),
        ("sites", "0002_alter_domain_unique"),
        ("meta_prn", "0031_alter_historicaloffschedule_options_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalEgfrNotification",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                (
                    "creatinine_value",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        verbose_name="Creatinine <u>level</u>",
                    ),
                ),
                (
                    "creatinine_units",
                    models.CharField(
                        choices=[
                            ("mg/dL", "mg/dL"),
                            ("umol/L", "μmol/L (micromoles/L)"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=15,
                        verbose_name="Units (creatinine)",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow, verbose_name="Report Date and Time"
                    ),
                ),
                ("creatinine_date", models.DateField(verbose_name="Creatinine result date")),
                (
                    "egfr_percent_change",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="Copy from RFT result in the eGFR section",
                        max_digits=10,
                        null=True,
                        verbose_name="Percent change from baseline",
                    ),
                ),
                (
                    "narrative",
                    models.TextField(blank=True, null=True, verbose_name="Narrative"),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical eGFR Notification",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="EgfrNotification",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "creatinine_value",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        verbose_name="Creatinine <u>level</u>",
                    ),
                ),
                (
                    "creatinine_units",
                    models.CharField(
                        choices=[
                            ("mg/dL", "mg/dL"),
                            ("umol/L", "μmol/L (micromoles/L)"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=15,
                        verbose_name="Units (creatinine)",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow, verbose_name="Report Date and Time"
                    ),
                ),
                ("creatinine_date", models.DateField(verbose_name="Creatinine result date")),
                (
                    "egfr_percent_change",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="Copy from RFT result in the eGFR section",
                        max_digits=10,
                        null=True,
                        verbose_name="Percent change from baseline",
                    ),
                ),
                (
                    "narrative",
                    models.TextField(blank=True, null=True, verbose_name="Narrative"),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "eGFR Notification",
                "verbose_name_plural": "eGFR Notifications",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "unique_together": {("subject_identifier", "creatinine_date")},
            },
            managers=[
                (
                    "objects",
                    edc_action_item.models.action_model_mixin.ActionIdentifierModelManager(),
                ),
            ],
        ),
    ]



import os
from importlib.metadata import PackageNotFoundError, version

from .celery import app as celery_app

try:
    __version__ = version(os.getcwd().split(os.sep)[-1])
except PackageNotFoundError:
    __version__ = None


__all__ = ["celery_app", "__version__"]

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "meta_edc.settings.debug")



# Generated by Django 3.2.6 on 2021-08-23 20:53

import edc_vitals.models.fields.blood_pressure
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0027_auto_20210804_0438"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="dia_blood_pressure_avg",
            field=models.IntegerField(
                blank=True,
                null=True,
                verbose_name="Blood pressure: diastolic (average)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="dia_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="dia_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="sys_blood_pressure_avg",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Blood pressure: systolic (average)"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="sys_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="sys_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="dia_blood_pressure_avg",
            field=models.IntegerField(
                blank=True,
                null=True,
                verbose_name="Blood pressure: diastolic (average)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="dia_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="dia_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="sys_blood_pressure_avg",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Blood pressure: systolic (average)"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="sys_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="sys_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="dia_blood_pressure_avg",
            field=models.IntegerField(
                blank=True,
                null=True,
                verbose_name="Blood pressure: diastolic (average)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="dia_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="dia_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="sys_blood_pressure_avg",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Blood pressure: systolic (average)"
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="sys_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="sys_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="dia_blood_pressure_avg",
            field=models.IntegerField(
                blank=True,
                null=True,
                verbose_name="Blood pressure: diastolic (average)",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="dia_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="dia_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="sys_blood_pressure_avg",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Blood pressure: systolic (average)"
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="sys_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="sys_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="dia_blood_pressure_avg",
            field=models.IntegerField(
                blank=True,
                null=True,
                verbose_name="Blood pressure: diastolic (average)",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="dia_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="dia_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="sys_blood_pressure_avg",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Blood pressure: systolic (average)"
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="sys_blood_pressure_one",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="sys_blood_pressure_two",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="part_three_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 3 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="part_two_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 2 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="part_three_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 3 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="part_two_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 2 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="part_three_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 3 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="part_two_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 2 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="part_three_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 3 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="part_two_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 2 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="part_three_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 3 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="part_two_report_datetime",
            field=models.DateTimeField(
                help_text="Date and time of report.",
                null=True,
                verbose_name="Part 2 report date and time",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-07-22 01:11
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_action_item.identifiers import ActionIdentifier
from edc_constants.constants import CLOSED, HIGH_PRIORITY
from tqdm import tqdm

from meta_subject.constants import MISSED_VISIT_ACTION


def update_missing_action_items_for_missed_visits(apps, schema_editor):
    actiontype_model_cls = apps.get_model("edc_action_item.actiontype")
    actionitem_model_cls = apps.get_model("edc_action_item.actionitem")
    subjectvisitmissed_model_cls = apps.get_model("meta_subject.subjectvisitmissed")
    subjectvisit_model_cls = apps.get_model("meta_subject.subjectvisit")
    try:
        action_type = actiontype_model_cls.objects.get(name=MISSED_VISIT_ACTION)
    except ObjectDoesNotExist:
        pass
    else:
        total = subjectvisitmissed_model_cls.objects.filter(
            action_identifier__isnull=True
        ).count()
        for obj in tqdm(
            subjectvisitmissed_model_cls.objects.filter(action_identifier__isnull=True),
            total=total,
        ):
            subject_visit = subjectvisit_model_cls.objects.get(id=obj.subject_visit_id)
            action_item = actionitem_model_cls.objects.create(
                subject_identifier=subject_visit.subject_identifier,
                action_identifier=ActionIdentifier(site_id=subject_visit.site_id).identifier,
                report_datetime=subject_visit.report_datetime,
                action_type=action_type,
                reference_model="meta_subject.subjectvisitmissed",
                linked_to_reference=True,
                priority=HIGH_PRIORITY,
                status=CLOSED,
                auto_created=True,
                site_id=subject_visit.site_id,
            )
            obj.action_identifier = action_item.action_identifier
            obj.save_base(update_fields=["action_identifier"])


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0130_auto_20220720_0216"),
    ]

    operations = [migrations.RunPython(update_missing_action_items_for_missed_visits)]



from django.forms.widgets import Input


class SliderWidget(Input):
    input_type = "range"
    class_type = "slider"
    units = "%"
    step = 1
    template_name = "meta_subject/widgets/slider.html"

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        if attrs.get("step"):
            context["widget"]["step"] = str(attrs.get("step"))
        else:
            context["widget"]["step"] = str(self.step)
        context["widget"]["type"] = self.input_type
        context["widget"]["class"] = self.class_type
        context["widget"]["units"] = self.units
        context["widget"]["min"] = attrs.get("min")
        context["widget"]["max"] = attrs.get("max")
        return context

    class Media:
        css = {"all": ("meta_subject/slider.css",)}



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import HospitalizationModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_ae_admin
from ..forms import HospitalizationForm
from ..models import Hospitalization


@admin.register(Hospitalization, site=meta_ae_admin)
class HospitalizationAdmin(
    SiteModelAdminMixin, HospitalizationModelAdminMixin, SimpleHistoryAdmin
):
    form = HospitalizationForm



from django.contrib import admin
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ...admin_site import meta_reports_admin
from ...models import UnattendedThreeInRow2


@admin.register(UnattendedThreeInRow2, site=meta_reports_admin)
class UnattendedThreeInRow2Admin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ["site", "subject_identifier"]

    list_display = [
        "dashboard",
        "subject",
        "first_value",
        "second_value",
        "third_value",
        "interval_days",
        "from_now_days",
        "site",
        "missed_count",
        "created",
    ]

    list_filter = [
        "missed_count",
        ScheduleStatusListFilter,
        "first_value",
        "second_value",
        "third_value",
    ]

    search_fields = ["id", "subject_identifier", "first_value", "second_value", "third_value"]

    @admin.display(description="Subject", ordering="subject_identifier")
    def subject(self, obj):
        return obj.subject_identifier



from django.contrib import admin
from edc_microscopy.modeladmin_mixins import MalariaTestModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import MalariaTestForm
from ..models import MalariaTest
from .modeladmin import CrfModelAdminMixin


@admin.register(MalariaTest, site=meta_subject_admin)
class MalariaTestAdmin(MalariaTestModelAdminMixin, CrfModelAdminMixin, SimpleHistoryAdmin):
    form = MalariaTestForm



# Generated by Django 3.2.4 on 2021-07-19 01:10

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_consent", "0004_auto_20210624_0225"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectconsent",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "consent_datetime",
                "ordering": ("created",),
                "verbose_name": "Subject Consent",
                "verbose_name_plural": "Subject Consents",
            },
        ),
    ]



# Generated by Django 5.1.2 on 2025-01-15 23:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0028_historicalsubjectconsentv1ext_assessment_score_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsentv1ext",
            name="agrees_to_extension",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="See above for the definition of extended followup.",
                max_length=15,
                verbose_name="Does the participant give consent to extend clinic followup as per the protocol amendment?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsentv1ext",
            name="agrees_to_extension",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="See above for the definition of extended followup.",
                max_length=15,
                verbose_name="Does the participant give consent to extend clinic followup as per the protocol amendment?",
            ),
        ),
    ]



from django.db import models
from django_db_views.db_view import DBView
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions

from .view_definition import get_view_definition


class GlucoseSummary(QaReportModelMixin, DBView):

    fbg_value = models.DecimalField(max_digits=8, decimal_places=2, null=True)

    fbg_units = models.CharField(max_length=15, null=True)

    fbg_datetime = models.DateTimeField(null=True)

    ogtt_value = models.DecimalField(max_digits=8, decimal_places=2, null=True)

    ogtt_units = models.CharField(max_length=15, null=True)

    ogtt_datetime = models.DateTimeField(null=True)

    fasted = models.CharField(max_length=15, null=True)

    fasting_duration_delta = models.DurationField(
        verbose_name="Fasting duration (hrs)", null=True
    )

    report_datetime = models.DateTimeField(null=True)

    visit_code = models.CharField(max_length=25)

    visit_code_sequence = models.IntegerField()

    appointment_id = models.UUIDField(null=True)

    offstudy_datetime = models.DateTimeField(null=True)

    source = models.CharField(max_length=35, null=True)

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "meta_reports_glucosesummaryview"
        verbose_name = "Glucose Summary"
        verbose_name_plural = "Glucose Summary"
        default_permissions = qa_reports_permissions



from django.db import models
from edc_crf.model_mixins import CrfStatusModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin, requisition_fk_options
from edc_lab_panel.panels import hba1c_panel
from edc_lab_results import BLOOD_RESULTS_HBA1C_ACTION
from edc_lab_results.model_mixins import BloodResultsModelMixin, Hba1cModelMixin
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsHba1c(
    CrfWithActionModelMixin,
    Hba1cModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    CrfStatusModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_HBA1C_ACTION
    lab_panel = hba1c_panel

    requisition = models.ForeignKey(
        limit_choices_to={"panel__name": hba1c_panel.name}, **requisition_fk_options
    )

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: HbA1c"
        verbose_name_plural = "Blood Results: HbA1c"



from __future__ import annotations

from copy import deepcopy
from datetime import datetime
from decimal import Decimal
from zoneinfo import ZoneInfo

import time_machine
from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.test import TestCase
from edc_action_item import site_action_items
from edc_action_item.models import ActionItem
from edc_constants.constants import BLACK, MALE
from edc_egfr.calculators import egfr_percent_change
from edc_lab.models import Panel
from edc_lab_results import BLOOD_RESULTS_EGFR_ACTION, BLOOD_RESULTS_RFT_ACTION
from edc_registration import get_registered_subject_model_cls
from edc_reportable import MICROMOLES_PER_LITER, MILLIGRAMS_PER_DECILITER
from edc_utils import age, get_utcnow
from edc_utils.round_up import round_half_away_from_zero
from edc_visit_schedule.constants import OFFSCHEDULE_ACTION

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_screening.tests.options import now
from meta_subject.forms.blood_results.blood_results_rft_form import (
    BloodResultsRftFormValidator,
)
from meta_subject.models import (
    BloodResultsRft,
    EgfrDropNotification,
    SubjectRequisition,
)


@time_machine.travel(datetime(2019, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
class TestEgfr(MetaTestCaseMixin, TestCase):
    def setUp(self):
        self.subject_visit = self.get_subject_visit(screening_datetime=get_utcnow())
        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=self.subject_visit,
            panel=panel,
            requisition_datetime=self.subject_visit.report_datetime,
        )
        self.data = dict(
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )

    def tearDown(self) -> None:
        EgfrDropNotification.objects.all().delete()

    def test_model(self):
        data = deepcopy(self.data)
        data.update(creatinine_value=1.1, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj = BloodResultsRft.objects.create(**data)
        self.assertIsNotNone(obj.egfr_value)
        self.assertIsNotNone(obj.egfr_units)

    def test_ok(self):
        traveller = time_machine.travel(self.subject_visit.report_datetime)
        traveller.start()
        data = dict(
            subject_visit=self.subject_visit,
            report_datetime=get_utcnow(),
        )
        form = BloodResultsRftFormValidator(cleaned_data=data, model=BloodResultsRft)
        try:
            form.validate()
        except forms.ValidationError:
            pass
        self.assertEqual({}, form._errors)

    def test_egfr_drop(self):
        data = deepcopy(self.data)
        data.update(creatinine_value=1.1, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj_1000 = BloodResultsRft.objects.create(**data)
        obj_1000.refresh_from_db()
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=subject_visit,
            panel=panel,
            requisition_datetime=subject_visit.report_datetime,
        )
        data = dict(
            subject_visit=subject_visit,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        data.update(creatinine_value=1.5, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj_2000 = BloodResultsRft.objects.create(**data)
        obj_2000.refresh_from_db()
        self.assertGreaterEqual(
            float(obj_1000.egfr_value - obj_2000.egfr_value), 0.20 * float(obj_1000.egfr_value)
        )
        self.assertEqual(
            round_half_away_from_zero(float(obj_2000.egfr_drop_value), 2),
            round_half_away_from_zero(
                egfr_percent_change(obj_2000.egfr_value, obj_1000.egfr_value), 2
            ),
        )

    def test_egfr_drop_percent_drop_cannot_be_negative(self):
        data = deepcopy(self.data)
        data.update(creatinine_value=1.1, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj_1000 = BloodResultsRft.objects.create(**data)
        obj_1000.refresh_from_db()
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit.refresh_from_db()
        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=subject_visit,
            panel=panel,
            requisition_datetime=subject_visit.report_datetime,
        )
        data = dict(
            subject_visit=subject_visit,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        data.update(creatinine_value=0.58, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj_2000 = BloodResultsRft.objects.create(**data)
        obj_2000.refresh_from_db()
        self.assertLess(float(obj_1000.egfr_value - obj_2000.egfr_value), 0.0)
        self.assertEqual(float(obj_2000.egfr_drop_value), 0.0)

    def test_egfr_below_45(self):
        data = deepcopy(self.data)
        data.update(creatinine_value=1.1, creatinine_units=MILLIGRAMS_PER_DECILITER)
        BloodResultsRft.objects.create(**data)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=subject_visit,
            panel=panel,
            requisition_datetime=subject_visit.report_datetime,
        )
        data = dict(
            subject_visit=subject_visit,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        data.update(creatinine_value=2.0, creatinine_units=MILLIGRAMS_PER_DECILITER)
        obj_2000 = BloodResultsRft.objects.create(**data)
        obj_2000.save()
        obj_2000.refresh_from_db()

        self.assertTrue(obj_2000.egfr_value < 45)

        self.assertIn(BLOOD_RESULTS_EGFR_ACTION, site_action_items.registry)
        self.assertTrue(
            site_action_items.registry.get(BLOOD_RESULTS_EGFR_ACTION).reference_model,
            "meta_subject.bloodresultrft",
        )
        self.assertTrue(
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=OFFSCHEDULE_ACTION,
            )
        )
        self.assertTrue(
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=BLOOD_RESULTS_RFT_ACTION,
                action_identifier=obj_2000.action_identifier,
            )
        )

    def test_egfr_ethnicity(self):
        subject_visit = self.get_subject_visit(
            ethnicity=BLACK, age_in_years=60, appt_datetime=now, gender=MALE
        )
        rs = get_registered_subject_model_cls().objects.get(
            subject_identifier=subject_visit.subject_identifier
        )
        self.assertEqual(rs.ethnicity, BLACK)
        self.assertEqual(
            age(rs.dob, reference_dt=subject_visit.report_datetime.date()).years, 60
        )

        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=subject_visit,
            panel=panel,
            requisition_datetime=subject_visit.report_datetime,
            drawn_datetime=subject_visit.report_datetime,
        )
        data = dict(
            subject_visit=subject_visit,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            report_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        data = deepcopy(data)
        data.update(creatinine_value=152.0, creatinine_units=MICROMOLES_PER_LITER)
        obj_1000 = BloodResultsRft.objects.create(**data)
        obj_1000.refresh_from_db()
        self.assertEqual(round_half_away_from_zero(obj_1000.egfr_value, 3), Decimal("49.019"))
        self.assertEqual(round_half_away_from_zero(obj_1000.egfr_value, 2), Decimal("49.02"))
        self.assertEqual(round_half_away_from_zero(obj_1000.egfr_value, 1), Decimal("49.0"))
        self.assertEqual(round_half_away_from_zero(obj_1000.egfr_value), Decimal("49"))

        subject_visit = self.get_next_subject_visit(subject_visit)
        subject_visit.refresh_from_db()
        panel = Panel.objects.get(name="chemistry_rft")
        requisition = SubjectRequisition.objects.create(
            subject_visit=subject_visit,
            panel=panel,
            requisition_datetime=subject_visit.report_datetime,
        )
        data = dict(
            subject_visit=subject_visit,
            requisition=requisition,
            assay_datetime=requisition.requisition_datetime,
            report_datetime=requisition.requisition_datetime,
            site=Site.objects.get(id=settings.SITE_ID),
        )
        data.update(creatinine_value=150.8, creatinine_units=MICROMOLES_PER_LITER)
        obj_2000 = BloodResultsRft.objects.create(**data)
        obj_2000.refresh_from_db()
        self.assertEqual(round_half_away_from_zero(obj_2000.egfr_value, 3), Decimal("49.492"))
        self.assertEqual(round_half_away_from_zero(obj_2000.egfr_value, 2), Decimal("49.49"))
        self.assertEqual(round_half_away_from_zero(obj_2000.egfr_value, 1), Decimal("49.5"))
        self.assertEqual(round_half_away_from_zero(obj_2000.egfr_value), Decimal("49"))



import re

from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR
from edc_pharmacy.form_validators import (
    StudyMedicationFormValidator as BaseStudyMedicationFormValidator,
)
from edc_pharmacy.models import Stock
from edc_visit_schedule.constants import DAY1

from ..models import StudyMedication


class StudyMedicationFormValidator(BaseStudyMedicationFormValidator):
    def clean(self):
        super().clean()
        self.validate_half_dose_at_baseline()
        self.validate_stock_codes_are_dispensed()

    def validate_half_dose_at_baseline(self):
        """Require 1000mg dose at baseline"""
        try:
            subject_visit = (
                self.cleaned_data.get("subject_visit") or self.instance.related_visit
            )
        except AttributeError:
            self.raise_validation_error("Subject visit is required", INVALID_ERROR)
        else:
            if subject_visit.visit_code == DAY1 and subject_visit.visit_code_sequence == 0:
                if (
                    self.cleaned_data.get("dosage_guideline")
                    and self.cleaned_data.get("dosage_guideline").dose != 1000
                ):
                    raise forms.ValidationError(
                        {"dosage_guideline": "Invalid. Expected 1000mg/day at baseline"}
                    )

    def validate_stock_codes_are_dispensed(self):
        if self.cleaned_data.get("stock_codes"):
            # pattern = re.compile("^([A-Z0-9]{6})(,[A-Z0-9]{6})*$")
            pattern = re.compile("^([A-Z0-9]{6})(\r\n[A-Z0-9]{6})*$")
            if not pattern.match(self.cleaned_data.get("stock_codes")):
                raise forms.ValidationError(
                    {
                        "stock_codes": (
                            "Invalid format. Enter one or more valid codes. "
                            "One code per line only. No commas, spaces, etc"
                        )
                    }
                )
            for stock_code in self.cleaned_data.get("stock_codes").split("\r\n"):
                try:
                    Stock.objects.get(
                        code=stock_code,
                        dispensed=True,
                        allocation__registered_subject__subject_identifier=(
                            self.subject_identifier
                        ),
                    )
                except Stock.DoesNotExist:
                    raise forms.ValidationError(
                        {
                            "stock_codes": (
                                f"Invalid. Got {stock_code}. "
                                "Either not allocated to this subject or not dispensed. "
                                "Please check the bottle or check with your pharmacist."
                            )
                        }
                    )


class StudyMedicationForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = StudyMedicationFormValidator

    class Meta:
        model = StudyMedication
        fields = "__all__"



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import NO, OTHER
from edc_crf.crf_form_validator import CrfFormValidator
from edc_form_validators import INVALID_ERROR
from edc_utils import formatted_date

from meta_ae.constants import HOSPITAL_CLINIC
from meta_prn.models import PregnancyNotification


class DeliveryFormValidator(CrfFormValidator):

    def clean(self):
        try:
            pregnancy_notification = PregnancyNotification.objects.get(
                subject_identifier=self.cleaned_data.get("subject_visit").subject_identifier
            )
        except ObjectDoesNotExist:
            raise forms.ValidationError(
                f"{PregnancyNotification._meta.verbose_name} not found."
            )

        self.required_if(
            NO, field="info_available", field_required="info_not_available_reason"
        )

        self.validate_informant()

        self.validate_delivery_date_with_upt(pregnancy_notification)

        self.validate_delivery_location()

    def validate_informant(self):
        self.required_if(OTHER, field="info_source", field_required="info_source_other")
        self.required_if(NO, field="info_source", field_required="informant_relation")
        self.validate_other_specify(
            field="informant_relation", other_specify_field="informant_relation_other"
        )

    def validate_delivery_date_with_upt(self, pregnancy_notification):
        """Delivery date must be after the UPT date"""
        if (
            self.cleaned_data.get("delivery_datetime")
            and self.cleaned_data.get("report_datetime")
            and self.cleaned_data.get("report_datetime").date()
            < self.cleaned_data.get("delivery_datetime").date()
        ):
            self.raise_validation_error(
                {"delivery_datetime": "Expected a date on or before the report date above"},
                INVALID_ERROR,
            )
        if (
            self.cleaned_data.get("delivery_datetime")
            and (
                pregnancy_notification.bhcg_date
                or pregnancy_notification.report_datetime.date()
            )
            > self.cleaned_data.get("delivery_datetime").date()
        ):
            dte = formatted_date(
                pregnancy_notification.bhcg_date
                or pregnancy_notification.report_datetime.date()
            )
            word = "UPT date" if pregnancy_notification.bhcg_date else "Pregnancy report date"
            self.raise_validation_error(
                {
                    "delivery_datetime": (
                        f"Expected a date after the {word} on the "
                        f"{pregnancy_notification._meta.verbose_name}. {word} was {dte}."
                    )
                },
                INVALID_ERROR,
            )

    def validate_delivery_location(self):
        self.required_if(
            HOSPITAL_CLINIC,
            field="delivery_location",
            field_required="delivery_location_name",
        )
        self.validate_other_specify(
            field="delivery_location", other_specify_field="delivery_location_other"
        )



from typing import Tuple

from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_adverse_event.forms import AeFollowupForm
from edc_adverse_event.modeladmin_mixins import (
    AdverseEventModelAdminMixin,
    NonAeInitialModelAdminMixin,
)
from edc_adverse_event.templatetags.edc_adverse_event_extras import (
    format_ae_followup_description,
    select_description_template,
)
from edc_constants.constants import NO, NOT_APPLICABLE, YES
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin


class AeReviewModelAdminMixin(
    ModelAdminSubjectDashboardMixin,
    NonAeInitialModelAdminMixin,
    AdverseEventModelAdminMixin,
    ActionItemModelAdminMixin,
):
    form = AeFollowupForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "ae_initial",
                    "report_datetime",
                    "outcome_date",
                    "outcome",
                    "ae_grade",
                    "relevant_history",
                    "followup",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "outcome": admin.VERTICAL,
        "followup": admin.VERTICAL,
        "ae_grade": admin.VERTICAL,
    }

    list_display: Tuple[str, ...] = (
        "identifier",
        "dashboard",
        "description",
        "initial_ae",
        "follow_up_reports",
        "user",
    )

    list_filter: Tuple[str, ...] = ("ae_grade", "followup", "outcome_date", "report_datetime")

    search_fields: Tuple[str, ...] = (
        "action_identifier",
        "ae_initial__subject_identifier",
        "ae_initial__action_identifier",
    )

    @staticmethod
    def description(obj=None):
        """Returns a formatted comprehensive description of the SAE
        combining multiple fields.
        """
        context = format_ae_followup_description({}, obj, 80)
        return render_to_string(select_description_template("aefollowup"), context)

    def status(self, obj):
        follow_up_reports = None
        if obj.followup == YES:
            try:
                ae_followup = self.model.objects.get(parent_action_item=obj.action_item)
            except ObjectDoesNotExist:
                pass
            else:
                follow_up_reports = self.follow_up_reports(ae_followup)
        elif obj.followup == NO and obj.ae_grade != NOT_APPLICABLE:
            follow_up_reports = self.initial_ae(obj)
        if follow_up_reports:
            return format_html(
                "{}. See {}",
                mark_safe(_(obj.get_outcome_display())),  # nosec B703, B308
                mark_safe(_(follow_up_reports)),  # nosec B703, B308
            )
        return obj.get_outcome_display()

    def follow_up_reports(self, obj):
        return super().follow_up_reports(obj.ae_initial)

    def initial_ae(self, obj):
        """Returns a shortened action identifier."""
        if obj.ae_initial:
            url_name = "_".join(obj.ae_initial._meta.label_lower.split("."))
            namespace = self.admin_site.name
            url = reverse(f"{namespace}:{url_name}_changelist")
            return format_html(
                '<a data-toggle="tooltip" title="{title}" '
                'href="{url}?q={action_identifier}">{identifier}</a>',
                title=_("go to ae initial report"),
                url=mark_safe(url),  # nosec B703, B308
                action_identifier=mark_safe(  # nosec B703, B308
                    obj.ae_initial.action_identifier
                ),
                identifier=mark_safe(obj.ae_initial.identifier),  # nosec B703, B308
            )
        return None
